import os
from keycloak import KeycloakAdmin


def keycloak_user_management_enabled():
    if "KEYCLOAK_USER_MANAGEMENT" in os.environ:
        return os.getenv("KEYCLOAK_USER_MANAGEMENT").lower() == "true"
    else:
        return False


def get_keycloak_admin():
    return KeycloakAdmin(
        server_url=os.getenv("KEYCLOAK_SERVER_URL"),
        username=os.getenv("KEYCLOAK_ADMIN_USERNAME"),
        password=os.getenv("KEYCLOAK_ADMIN_PASSWORD"),
        realm_name=os.getenv("KEYCLOAK_REALM_NAME"),
        client_secret_key=os.getenv("KEYCLOAK_CLIENT_SECRET_KEY"),
        verify=(os.getenv("KEYCLOAK_VERIFY").lower() == "true"),
    )


def create_user(user_data):
    if keycloak_user_management_enabled():
        keycloak_admin = get_keycloak_admin()
        keycloak_admin.create_user(
            {"username": user_data["username"], "credentials": [{"value": user_data["password"], "type": "password"}], "enabled": True}
        )


def update_user(user_data, original_username):
    if keycloak_user_management_enabled():
        if "password" in user_data and user_data["password"] or original_username != user_data["username"]:
            keycloak_admin = get_keycloak_admin()
            keycloak_user_id = keycloak_admin.get_user_id(original_username)
            if keycloak_user_id is not None:
                if original_username != user_data["username"]:
                    keycloak_admin.update_user(user_id=keycloak_user_id, payload={"username": user_data["username"]})

                if "password" in user_data and user_data["password"]:
                    keycloak_admin.set_user_password(user_id=keycloak_user_id, password=user_data["password"], temporary=False)


def delete_user(username):
    if keycloak_user_management_enabled():
        keycloak_admin = get_keycloak_admin()
        keycloak_user_id = keycloak_admin.get_user_id(username)
        if keycloak_user_id is not None:
            keycloak_admin.delete_user(user_id=keycloak_user_id)



from core.managers.log_manager import logger
from core.auth.base_authenticator import BaseAuthenticator


users = {"user": "user", "user2": "user", "admin": "admin", "customer": "customer"}


class TestAuthenticator(BaseAuthenticator):
    def get_authenticator_name(self):
        return "TestAuthenticator"

    def __str__(self):
        return f"Authenticator: {self.get_authenticator_name()} Users: {users}"

    def get_required_credentials(self):
        return ["username", "password"]

    def authenticate(self, credentials):
        logger.log_debug(f"TEST AUTH with {credentials}")
        if credentials is None:
            return BaseAuthenticator.generate_error()
        if "username" not in credentials or "password" not in credentials:
            return BaseAuthenticator.generate_error()
        if users[credentials["username"]] == credentials["password"]:
            return BaseAuthenticator.generate_jwt(credentials["username"])

        logger.store_auth_error_activity(f"Authentication failed with credentials: {str(credentials)}")
        return BaseAuthenticator.generate_error()



import requests
from core.managers.log_manager import logger


class CollectorsApi:
    def __init__(self, api_url: str, api_key: str):
        self.api_url = api_url[:-1] if api_url.endswith("/") else api_url
        self.api_key = api_key
        self.headers = {"Authorization": f"Bearer {self.api_key}"}

    def get_collectors_info(self):
        collector_endpoint = f"{self.api_url}/api/v1/collectors"
        logger.log_debug(f"Getting Collector Infos from: {collector_endpoint}")
        try:
            response = requests.get(collector_endpoint, headers=self.headers)
        except Exception as e:
            return str(e), 500

        return response.json(), response.status_code

    def refresh_collector(self, collector_type):
        response = requests.put(f"{self.api_url}/api/v1/collectors/{collector_type}", headers=self.headers)

        return response.status_code



from flask import request
from flask_restful import Resource

from core.managers import auth_manager
from core.managers.auth_manager import auth_required
from core.model import asset, notification_template, attribute
from shared.schema.attribute import AttributeType


class AssetGroups(Resource):
    @auth_required("MY_ASSETS_ACCESS")
    def get(self):
        search = None
        if "search" in request.args and request.args["search"]:
            search = request.args["search"]
        return asset.AssetGroup.get_all_json(auth_manager.get_user_from_jwt(), search)

    @auth_required("MY_ASSETS_CONFIG")
    def post(self):
        return "", asset.AssetGroup.add(auth_manager.get_user_from_jwt(), request.json)


class AssetGroup(Resource):
    @auth_required("MY_ASSETS_CONFIG")
    def put(self, group_id):
        asset.AssetGroup.update(auth_manager.get_user_from_jwt(), group_id, request.json)

    @auth_required("MY_ASSETS_CONFIG")
    def delete(self, group_id):
        return asset.AssetGroup.delete(auth_manager.get_user_from_jwt(), group_id)


class NotificationTemplates(Resource):
    @auth_required("MY_ASSETS_CONFIG")
    def get(self):
        search = None
        if "search" in request.args and request.args["search"]:
            search = request.args["search"]
        return notification_template.NotificationTemplate.get_all_json(auth_manager.get_user_from_jwt(), search)

    @auth_required("MY_ASSETS_CONFIG")
    def post(self):
        return "", notification_template.NotificationTemplate.add(auth_manager.get_user_from_jwt(), request.json)


class NotificationTemplate(Resource):
    @auth_required("MY_ASSETS_CONFIG")
    def put(self, template_id):
        notification_template.NotificationTemplate.update(auth_manager.get_user_from_jwt(), template_id, request.json)

    @auth_required("MY_ASSETS_CONFIG")
    def delete(self, template_id):
        return notification_template.NotificationTemplate.delete(auth_manager.get_user_from_jwt(), template_id)


class Assets(Resource):
    @auth_required("MY_ASSETS_ACCESS")
    def get(self, group_id):
        search = None
        sort = None
        vulnerable = None
        if "search" in request.args and request.args["search"]:
            search = request.args["search"]
        if "sort" in request.args and request.args["sort"]:
            sort = request.args["sort"]
        if "vulnerable" in request.args and request.args["vulnerable"]:
            vulnerable = request.args["vulnerable"]
        return asset.Asset.get_all_json(auth_manager.get_user_from_jwt(), group_id, search, sort, vulnerable)

    @auth_required("MY_ASSETS_CREATE")
    def post(self, group_id):
        return "", asset.Asset.add(auth_manager.get_user_from_jwt(), group_id, request.json)


class Asset(Resource):
    @auth_required("MY_ASSETS_CREATE")
    def put(self, group_id, asset_id):
        asset.Asset.update(auth_manager.get_user_from_jwt(), group_id, asset_id, request.json)

    @auth_required("MY_ASSETS_CREATE")
    def delete(self, group_id, asset_id):
        return asset.Asset.delete(auth_manager.get_user_from_jwt(), group_id, asset_id)


class AssetVulnerability(Resource):
    @auth_required("MY_ASSETS_CREATE")
    def put(self, group_id, asset_id, vulnerability_id):
        return asset.Asset.solve_vulnerability(
            auth_manager.get_user_from_jwt(),
            group_id,
            asset_id,
            vulnerability_id,
            request.json["solved"],
        )


class GetAttributeCPE(Resource):
    @auth_required("MY_ASSETS_CREATE")
    def get(self):
        cpe = attribute.Attribute.find_by_type(AttributeType.CPE)
        return cpe.id


class AttributeCPEEnums(Resource):
    @auth_required("MY_ASSETS_CREATE")
    def get(self):
        cpe = attribute.Attribute.find_by_type(AttributeType.CPE)
        search = None
        offset = 0
        limit = 10
        if "search" in request.args and request.args["search"]:
            search = request.args["search"]
        if "offset" in request.args and request.args["offset"]:
            offset = request.args["offset"]
        if "limit" in request.args and request.args["limit"]:
            limit = request.args["limit"]
        return attribute.AttributeEnum.get_for_attribute_json(cpe.id, search, offset, limit)


def initialize(api):
    api.add_resource(AssetGroups, "/api/v1/my-assets/asset-groups")
    api.add_resource(AssetGroup, "/api/v1/my-assets/asset-groups/<string:group_id>")

    api.add_resource(NotificationTemplates, "/api/v1/my-assets/asset-notification-templates")
    api.add_resource(
        NotificationTemplate,
        "/api/v1/my-assets/asset-notification-templates/<int:template_id>",
    )

    api.add_resource(Assets, "/api/v1/my-assets/asset-groups/<string:group_id>/assets")
    api.add_resource(Asset, "/api/v1/my-assets/asset-groups/<string:group_id>/assets/<int:asset_id>")

    api.add_resource(
        AssetVulnerability,
        "/api/v1/my-assets/asset-groups/<string:group_id>/assets/<int:asset_id>/vulnerabilities/<int:vulnerability_id>",
    )

    api.add_resource(GetAttributeCPE, "/api/v1/my-assets/attributes/cpe")
    api.add_resource(AttributeCPEEnums, "/api/v1/my-assets/attributes/cpe/enums")



__all__ = [
    "api_manager",
    "asset_manager",
    "audit_manager",
    "auth_manager",
    "bots_manager",
    "collectors_manager",
    "db_manager",
    "external_auth_manager",
    "log_manager",
    "presenters_manager",
    "publishers_manager",
    "remote_manager",
    "sse_manager",
    "tagcloud_manager",
    "time_manager",
]



from flask_restful import Resource, reqparse
from flask import request

from core.managers import publishers_manager
from core.managers.auth_manager import auth_required, api_key_required
from core.model import publishers_node, publisher_preset


class PublisherPresets(Resource):
    @auth_required("CONFIG_PUBLISHER_PRESET_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return publisher_preset.PublisherPreset.get_all_json(search)

    @api_key_required
    def post(self):
        parser = reqparse.RequestParser()
        parser.add_argument("api_key")
        parser.add_argument("collector_type")
        parameters = parser.parse_args()
        return publisher_preset.PublisherPreset.get_all_for_publisher_json(parameters)


class PublisherPreset(Resource):
    @auth_required("CONFIG_PUBLISHER_PRESET_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return publisher_preset.PublisherPreset.get_all_json(search)

    @auth_required("CONFIG_PUBLISHER_PRESET_CREATE")
    def post(self):
        publishers_manager.add_publisher_preset(request.json)

    @auth_required("CONFIG_PUBLISHER_PRESET_UPDATE")
    def put(self, id):
        publisher_preset.PublisherPreset.update(id, request.json)

    @auth_required("CONFIG_PUBLISHER_PRESET_DELETE")
    def delete(self, id):
        return publisher_preset.PublisherPreset.delete(id)


class PublishersNode(Resource):
    @auth_required("CONFIG_PUBLISHERS_NODE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return publishers_node.PublishersNode.get_all_json(search)

    @auth_required("CONFIG_PUBLISHERS_NODE_CREATE")
    def post(self):
        return "", publishers_manager.add_publishers_node(request.json)

    @auth_required("CONFIG_PUBLISHERS_NODE_UPDATE")
    def put(self, id):
        publishers_manager.update_publishers_node(id, request.json)

    @auth_required("CONFIG_PUBLISHERS_NODE_DELETE")
    def delete(self, id):
        return publishers_node.PublishersNode.delete(id)


def initialize(api):
    api.add_resource(PublishersNode, "/api/publishers/nodes", "/api/publishers/node", "/api/publishers/node/<id>")

    api.add_resource(PublisherPresets, "/api/publishers/presets")
    api.add_resource(PublisherPreset, "/api/publishers/preset", "/api/publishers/preset/<id>")



from marshmallow import fields, post_load
from sqlalchemy import func, or_, orm

from core.managers.db_manager import db
from core.model.permission import Permission
from shared.schema.role import RoleSchemaBase, PermissionIdSchema, RolePresentationSchema


class NewRoleSchema(RoleSchemaBase):
    permissions = fields.Nested(PermissionIdSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return Role(**data)


class Role(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), unique=True, nullable=False)
    description = db.Column(db.String())
    permissions = db.relationship(Permission, secondary="role_permission", back_populates="roles")

    def __init__(self, id, name, description, permissions):
        self.id = None
        self.name = name
        self.description = description
        self.permissions = []
        for permission in permissions:
            self.permissions.append(Permission.find(permission.id))

        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-account-arrow-right"

    @classmethod
    def find(cls, role_id):
        return cls.query.get(role_id)

    @classmethod
    def find_by_name(cls, role_name):
        return cls.query.filter_by(name=role_name).first()

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(Role.name)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(Role.name).like(search_string),
                    func.lower(Role.description).like(search_string),
                )
            )

        return query.order_by(db.asc(Role.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        roles, count = cls.get(search)
        roles_schema = RolePresentationSchema(many=True)
        return {"total_count": count, "items": roles_schema.dump(roles)}

    @classmethod
    def add_new(cls, data):
        new_role_schema = NewRoleSchema()
        role = new_role_schema.load(data)
        db.session.add(role)
        db.session.commit()

    def get_permissions(self):
        all_permissions = set()
        for permission in self.permissions:
            all_permissions.add(permission.id)

        return all_permissions

    @classmethod
    def update(cls, role_id, data):
        schema = NewRoleSchema()
        updated_role = schema.load(data)
        role = cls.query.get(role_id)
        role.name = updated_role.name
        role.description = updated_role.description
        role.permissions = updated_role.permissions
        db.session.commit()

    @classmethod
    def delete(cls, id):
        role = cls.query.get(id)
        db.session.delete(role)
        db.session.commit()


class RolePermission(db.Model):
    role_id = db.Column(db.Integer, db.ForeignKey("role.id"), primary_key=True)
    permission_id = db.Column(db.String, db.ForeignKey("permission.id"), primary_key=True)



from flask_jwt_extended import create_access_token

from core.managers.log_manager import logger
from core.model.token_blacklist import TokenBlacklist
from core.model.user import User


class BaseAuthenticator:
    def get_required_credentials(self):
        return []

    def get_authenticator_name(self):
        return ""

    def __str__(self):
        return f"Authenticator: {self.get_authenticator_name()}"

    def authenticate(self, credentials):
        return BaseAuthenticator.generate_error()

    def refresh(self, user):
        return BaseAuthenticator.generate_jwt(user.username)

    @staticmethod
    def logout(token):
        if token is not None:
            TokenBlacklist.add(token)

    @staticmethod
    def initialize(app):
        pass

    @staticmethod
    def generate_error():
        return {"error": "Authentication failed"}, 401

    @staticmethod
    def generate_jwt(username):
        if user := User.find(username):
            logger.store_user_activity(user, "LOGIN", "Successful")
            access_token = create_access_token(
                identity=user.username,
                additional_claims={
                    "user_claims": {
                        "id": user.id,
                        "name": user.name,
                        "organization_name": user.get_current_organization_name(),
                        "permissions": user.get_permissions(),
                    }
                },
            )

            return {"access_token": access_token}, 200

        logger.store_auth_error_activity(f"User doesn't exists: {username}")
        return BaseAuthenticator.generate_error()



from marshmallow import post_load

from core.managers.db_manager import db
from shared.schema.parameter_value import ParameterValueSchema


class NewParameterValueSchema(ParameterValueSchema):
    @post_load
    def make_parameter_value(self, data, **kwargs):
        return ParameterValue(**data)


class ParameterValue(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    value = db.Column(db.String(), nullable=False)

    parameter_id = db.Column(db.Integer, db.ForeignKey("parameter.id"))
    parameter = db.relationship("Parameter")

    def __init__(self, value, parameter):
        self.id = None
        self.value = value
        self.parameter_id = parameter.id



from flask import request
import json
from flask_restful import Resource, reqparse
from datetime import datetime, timedelta

from core.managers import sse_manager, bots_manager
from core.managers.log_manager import logger
from core.managers.auth_manager import api_key_required
from core.model import bot_preset, news_item, word_list, bots_node


class BotPresetsForBots(Resource):
    @api_key_required
    def post(self):
        parser = reqparse.RequestParser()
        parser.add_argument("api_key")
        parser.add_argument("bot_type")
        parameters = parser.parse_args()
        return bot_preset.BotPreset.get_all_for_bot_json(parameters)


class BotGroupAction(Resource):
    @api_key_required
    def put(self):
        response, osint_source_ids, code = news_item.NewsItemAggregate.group_action(request.json, None)
        sse_manager.news_items_updated()
        if len(osint_source_ids) > 0:
            sse_manager.remote_access_news_items_updated(osint_source_ids)
        return response, code


class NewsItemData(Resource):
    @api_key_required
    def get(self):
        try:
            limit = datetime.strftime(datetime.now() - timedelta(weeks=1), "%d.%m.%Y - %H:%M")
            if "limit" in request.args:
                limit = request.args["limit"]
        except Exception:
            logger.log_debug_trace("GET /api/v1/bots/news-item-data failed")
            return "", 400

        return news_item.NewsItemData.get_all_news_items_data(limit)


class BotsNode(Resource):
    @api_key_required
    def get(self, node_id):
        return bots_node.BotsNode.get_json_by_id(node_id)

    @api_key_required
    def put(self, node_id):
        return bots_manager.update_bots_node(node_id, request.json)

    @api_key_required
    def post(self):
        return bots_manager.add_bots_node(request.json)

    @api_key_required
    def delete(self, node_id):
        bots_node.BotsNode.delete(node_id)


class UpdateNewsItemAttributes(Resource):
    @api_key_required
    def put(self, news_item_data_id):
        news_item.NewsItemData.update_news_item_attributes(news_item_data_id, request.json)


class UpdateNewsItemTags(Resource):
    @api_key_required
    def put(self, news_item_data_id):
        news_item.NewsItemData.update_news_item_tags(news_item_data_id, request.json)


class GetNewsItemsAggregate(Resource):
    @api_key_required
    def get(self, group_id):

        resp_str = news_item.NewsItemAggregate.get_news_items_aggregate(group_id, request.json)
        return json.loads(resp_str)


class GetDefaultNewsItemsAggregate(Resource):
    @api_key_required
    def get(self):
        resp_str = news_item.NewsItemAggregate.get_default_news_items_aggregate(request.args.get("limit"))
        return json.loads(resp_str)


class Categories(Resource):
    @api_key_required
    def get(self, category_id):
        return word_list.WordListCategory.get_categories(category_id)

    @api_key_required
    def put(self, category_id):
        return word_list.WordList.add_word_list_category(category_id, request.json)


class Entries(Resource):
    @api_key_required
    def delete(self, category_id, entry_name):
        return word_list.WordListEntry.delete_entries(category_id, entry_name)

    @api_key_required
    def put(self, category_id, entry_name):
        return word_list.WordListEntry.update_word_list_entries(category_id, entry_name, request.json)


class BotsStatusUpdate(Resource):
    @api_key_required
    def get(self, node_id):
        collector = bots_node.BotsNode.get_by_id(node_id)
        if not collector:
            return "", 404

        try:
            collector.updateLastSeen()
        except Exception as ex:
            logger.log_debug(ex)
            return {}, 400

        return {}, 200


def initialize(api):
    api.add_resource(BotPresetsForBots, "/api/v1/bots/bots-presets")
    api.add_resource(NewsItemData, "/api/v1/bots/news-item-data")
    api.add_resource(
        UpdateNewsItemTags,
        "/api/v1/bots/news-item-data/<string:news_item_data_id>/tags",
    )
    api.add_resource(
        UpdateNewsItemAttributes,
        "/api/v1/bots/news-item-data/<string:news_item_data_id>/attributes",
    )
    api.add_resource(BotGroupAction, "/api/v1/bots/news-item-aggregates-group-action")
    api.add_resource(
        GetNewsItemsAggregate,
        "/api/v1/bots/news-item-aggregates-by-group/<string:group_id>",
    )
    api.add_resource(GetDefaultNewsItemsAggregate, "/api/v1/bots/news-item-aggregates")
    api.add_resource(Categories, "/api/v1/bots/word-list-categories/<int:category_id>")
    api.add_resource(
        Entries,
        "/api/v1/bots/word-list-categories/<int:category_id>/entries/<string:entry_name>",
    )
    api.add_resource(BotsNode, "/api/v1/bots/node/<string:node_id>", "/api/v1/bots/node")
    api.add_resource(BotsStatusUpdate, "/api/v1/bots/<string:node_id>")



from sqlalchemy import func, or_, orm, and_
from marshmallow import fields, post_load
from flask_sqlalchemy import BaseQuery

from core.managers.db_manager import db
from core.model.role import Role
from core.model.user import User
from shared.schema.role import RoleIdSchema
from shared.schema.user import UserIdSchema
from shared.schema.acl_entry import ACLEntrySchema, ACLEntryPresentationSchema, ItemType


class NewACLEntrySchema(ACLEntrySchema):
    users = fields.Nested(UserIdSchema, many=True)
    roles = fields.Nested(RoleIdSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return ACLEntry(**data)


class ACLEntry(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), unique=True, nullable=False)
    description = db.Column(db.String())

    item_type = db.Column(db.Enum(ItemType))
    item_id = db.Column(db.String(64))

    everyone = db.Column(db.Boolean, default=True)
    users = db.relationship("User", secondary="acl_entry_user")
    roles = db.relationship("Role", secondary="acl_entry_role")

    see = db.Column(db.Boolean)
    access = db.Column(db.Boolean)
    modify = db.Column(db.Boolean)

    def __init__(self, id, name, description, item_type, item_id, everyone, users, see, access, modify, roles):
        self.id = None
        self.name = name
        self.description = description
        self.item_type = item_type
        self.item_id = item_id
        self.everyone = everyone
        self.see = see
        self.access = access
        self.modify = modify
        self.users = [User.find_by_id(user.id) for user in users]
        self.roles = [Role.find(role.id) for role in roles]
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-lock-check"

    @classmethod
    def find(cls, id):
        return cls.query.get(id)

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(ACLEntry.name)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(ACLEntry.name).like(search_string),
                    func.lower(ACLEntry.description).like(search_string),
                )
            )

        return query.order_by(db.asc(ACLEntry.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        acls, count = cls.get(search)
        acl_schema = ACLEntryPresentationSchema(many=True)
        return {"total_count": count, "items": acl_schema.dump(acls)}

    @classmethod
    def add_new(cls, data):
        new_acl_schema = NewACLEntrySchema()
        acl = new_acl_schema.load(data)
        db.session.add(acl)
        db.session.commit()

    @classmethod
    def update(cls, acl_id, data):
        schema = NewACLEntrySchema()
        updated_acl = schema.load(data)
        acl = cls.query.get(acl_id)
        acl.name = updated_acl.name
        acl.description = updated_acl.description
        acl.item_type = updated_acl.item_type
        acl.item_id = updated_acl.item_id
        acl.everyone = updated_acl.everyone
        acl.see = updated_acl.see
        acl.access = updated_acl.access
        acl.modify = updated_acl.modify
        acl.users = updated_acl.users
        acl.roles = updated_acl.roles
        db.session.commit()

    @classmethod
    def delete(cls, id):
        acl = cls.query.get(id)
        db.session.delete(acl)
        db.session.commit()

    @classmethod
    def apply_query(cls, query: BaseQuery, user: User, see: bool, access: bool, modify: bool) -> BaseQuery:
        roles = [role.id for role in user.roles]

        query = query.outerjoin(
            ACLEntryUser,
            and_(
                ACLEntryUser.acl_entry_id == ACLEntry.id,
                ACLEntryUser.user_id == user.id,
            ),
        )

        query = query.outerjoin(ACLEntryRole, ACLEntryRole.acl_entry_id == ACLEntry.id)

        if see:
            return query.filter(
                or_(
                    ACLEntry.id is not None,
                    and_(
                        ACLEntry.see is True,
                        or_(
                            ACLEntry.everyone is True,
                            ACLEntryUser.user_id == user.id,
                            ACLEntryRole.role_id.in_(roles),
                        ),
                    ),
                )
            )

        if access:
            return query.filter(
                or_(
                    ACLEntry.id is not None,
                    and_(
                        ACLEntry.access is True,
                        or_(
                            ACLEntry.everyone is True,
                            ACLEntryUser.user_id == user.id,
                            ACLEntryRole.role_id.in_(roles),
                        ),
                    ),
                )
            )

        if modify:
            return query.filter(
                or_(
                    ACLEntry.id is not None,
                    and_(
                        ACLEntry.modify is True,
                        or_(
                            ACLEntry.everyone is True,
                            ACLEntryUser.user_id == user.id,
                            ACLEntryRole.role_id.in_(roles),
                        ),
                    ),
                )
            )

        return query.filter(
            or_(
                ACLEntry.id is not None,
                ACLEntry.everyone is True,
                ACLEntryUser.user_id == user.id,
                ACLEntryRole.role_id.in_(roles),
            )
        )


class ACLEntryUser(db.Model):
    acl_entry_id = db.Column(db.Integer, db.ForeignKey("acl_entry.id"), primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), primary_key=True)


class ACLEntryRole(db.Model):
    acl_entry_id = db.Column(db.Integer, db.ForeignKey("acl_entry.id"), primary_key=True)
    role_id = db.Column(db.Integer, db.ForeignKey("role.id"), primary_key=True)



from marshmallow import post_load
from sqlalchemy import func, or_, orm
import uuid

from core.managers.db_manager import db
from shared.schema.presenters_node import (
    PresentersNodeSchema,
    PresentersNodePresentationSchema,
)


class NewPresentersNodeSchema(PresentersNodeSchema):
    @post_load
    def make(self, data, **kwargs):
        return PresentersNode(**data)


class PresentersNode(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), unique=True, nullable=False)
    description = db.Column(db.String())

    api_url = db.Column(db.String(), nullable=False)
    api_key = db.Column(db.String(), nullable=False)

    presenters = db.relationship("Presenter", back_populates="node", cascade="all")

    def __init__(self, id, name, description, api_url, api_key):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.api_url = api_url
        self.api_key = api_key
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-server-network"

    @classmethod
    def exists_by_api_key(cls, api_key):
        return db.session.query(db.exists().where(PresentersNode.api_key == api_key)).scalar()

    @classmethod
    def get_by_api_key(cls, api_key):
        return cls.query.filter_by(api_key=api_key).first()

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(PresentersNode.name)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(PresentersNode.name).like(search_string),
                    func.lower(PresentersNode.description).like(search_string),
                )
            )

        return query.order_by(db.asc(PresentersNode.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        nodes, count = cls.get(search)
        node_schema = PresentersNodePresentationSchema(many=True)
        return {"total_count": count, "items": node_schema.dump(nodes)}

    @classmethod
    def add_new(cls, node_data, presenters):
        new_node_schema = NewPresentersNodeSchema()
        node = new_node_schema.load(node_data)
        node.presenters = presenters
        db.session.add(node)
        db.session.commit()

    @classmethod
    def update(cls, node_id, node_data, presenters):
        new_node_schema = NewPresentersNodeSchema()
        updated_node = new_node_schema.load(node_data)
        node = cls.query.get(node_id)
        node.name = updated_node.name
        node.description = updated_node.description
        node.api_url = updated_node.api_url
        node.api_key = updated_node.api_key
        for presenter in presenters:
            found = False
            for existing_presenter in node.presenters:
                if presenter.type == existing_presenter.type:
                    found = True
                    break

            if found is False:
                node.presenters.append(presenter)

        db.session.commit()

    @classmethod
    def delete(cls, node_id):
        node = cls.query.get(node_id)
        for presenter in node.presenters:
            if len(presenter.product_types) > 0:
                raise Exception("Presenters has mapped product types")

        db.session.delete(node)
        db.session.commit()



from authlib.integrations.flask_client import OAuth

from core.auth.base_authenticator import BaseAuthenticator


oauth = OAuth()


class OpenIDAuthenticator(BaseAuthenticator):
    @staticmethod
    def initialize(app):
        oauth.init_app(app)
        oauth.register(
            name="openid",
            client_id=app.config.get("OPENID_CLIENT_ID"),
            client_secret=app.config.get("OPENID_CLIENT_SECRET"),
            server_metadata_url=app.config.get("OPENID_METADAT_URL"),
        )

    def authenticate(self, credentials):
        oauth.openid.authorize_access_token()
        user = oauth.openid.userinfo()
        if user.preferred_username:
            print(user.preferred_username)
            return BaseAuthenticator.generate_jwt(user)

        return BaseAuthenticator.generate_error()

    @staticmethod
    def logout(token):
        BaseAuthenticator.logout(token)



from sqlalchemy import orm, func, or_
from marshmallow import post_load, fields

from core.managers.db_manager import db
from shared.schema.notification_template import (
    NotificationTemplatePresentationSchema,
    NotificationTemplateSchema,
    EmailRecipientSchema,
)


class NewEmailRecipientSchema(EmailRecipientSchema):
    @post_load
    def make(self, data, **kwargs):
        return EmailRecipient(**data)


class EmailRecipient(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    email = db.Column(db.String(), nullable=False)
    name = db.Column(db.String())

    notification_template_id = db.Column(db.Integer, db.ForeignKey("notification_template.id"))

    def __init__(self, email, name):
        self.id = None
        self.email = email
        self.name = name


class NewNotificationTemplateSchema(NotificationTemplateSchema):
    recipients = fields.Nested(NewEmailRecipientSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return NotificationTemplate(**data)


class NotificationTemplate(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    message_title = db.Column(db.String())
    message_body = db.Column(db.String())

    recipients = db.relationship("EmailRecipient", cascade="all, delete-orphan")

    organizations = db.relationship("Organization", secondary="notification_template_organization")

    def __init__(self, id, name, description, message_title, message_body, recipients):
        self.id = None
        self.name = name
        self.description = description
        self.message_title = message_title
        self.message_body = message_body
        self.recipients = recipients
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-email-outline"

    @classmethod
    def find(cls, id):
        group = cls.query.get(id)
        return group

    @classmethod
    def get(cls, search, organization):
        query = cls.query

        if organization is not None:
            query = query.join(
                NotificationTemplateOrganization,
                NotificationTemplate.id == NotificationTemplateOrganization.notification_template_id,
            )

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(NotificationTemplate.name).like(search_string),
                    func.lower(NotificationTemplate.description).like(search_string),
                )
            )

        return query.order_by(db.asc(NotificationTemplate.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, user, search):
        templates, count = cls.get(search, user.organizations[0])
        template_schema = NotificationTemplatePresentationSchema(many=True)
        return {"total_count": count, "items": template_schema.dump(templates)}

    @classmethod
    def add(cls, user, data):
        new_template_schema = NewNotificationTemplateSchema()
        template = new_template_schema.load(data)
        template.organizations = user.organizations
        db.session.add(template)
        db.session.commit()

    @classmethod
    def delete(cls, user, template_id):
        template = cls.query.get(template_id)
        if any(org in user.organizations for org in template.organizations):
            db.session.delete(template)
            db.session.commit()

    @classmethod
    def update(cls, user, template_id, data):
        new_template_schema = NewNotificationTemplateSchema()
        updated_template = new_template_schema.load(data)
        template = cls.query.get(template_id)
        if any(org in user.organizations for org in template.organizations):
            template.name = updated_template.name
            template.description = updated_template.description
            template.message_title = updated_template.message_title
            template.message_body = updated_template.message_body
            template.recipients = updated_template.recipients
            db.session.commit()


class NotificationTemplateOrganization(db.Model):
    notification_template_id = db.Column(db.Integer, db.ForeignKey("notification_template.id"), primary_key=True)
    organization_id = db.Column(db.Integer, db.ForeignKey("organization.id"), primary_key=True)



from marshmallow import fields, post_load
from sqlalchemy import func, or_, orm

from core.managers.db_manager import db
from core.model.role import Role
from core.model.permission import Permission
from core.model.organization import Organization
from shared.schema.user import (
    UserSchemaBase,
    UserProfileSchema,
    HotkeySchema,
    UserPresentationSchema,
)
from shared.schema.role import RoleIdSchema, PermissionIdSchema
from shared.schema.organization import OrganizationIdSchema


class NewUserSchema(UserSchemaBase):
    roles = fields.Nested(RoleIdSchema, many=True)
    permissions = fields.Nested(PermissionIdSchema, many=True)
    organizations = fields.Nested(OrganizationIdSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return User(**data)


class User(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(64), unique=True, nullable=False)
    name = db.Column(db.String(), nullable=False)
    password = db.Column(db.String(), nullable=True)

    organizations = db.relationship("Organization", secondary="user_organization")

    roles = db.relationship(Role, secondary="user_role")
    permissions = db.relationship(Permission, secondary="user_permission")

    profile_id = db.Column(db.Integer, db.ForeignKey("user_profile.id"))
    profile = db.relationship("UserProfile", cascade="all")

    def __init__(self, id, username, name, password, organizations, roles, permissions):
        self.id = None
        self.username = username
        self.name = name
        self.password = password
        self.organizations = []
        for organization in organizations:
            self.organizations.append(Organization.find(organization.id))

        self.roles = []
        for role in roles:
            self.roles.append(Role.find(role.id))

        self.permissions = []
        for permission in permissions:
            self.permissions.append(Permission.find(permission.id))

        self.profile = UserProfile(True, False, [])
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.username
        self.tag = "mdi-account"

    @classmethod
    def find(cls, username):
        return cls.query.filter_by(username=username).first()

    @classmethod
    def find_by_id(cls, user_id):
        return cls.query.get(user_id)

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(User.name)).all()

    @classmethod
    def get(cls, search, organization):
        query = cls.query

        if organization is not None:
            query = query.join(UserOrganization, User.id == UserOrganization.user_id)

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(User.name).like(search_string),
                    func.lower(User.username).like(search_string),
                )
            )

        return query.order_by(db.asc(User.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        users, count = cls.get(search, None)
        user_schema = UserPresentationSchema(many=True)
        return {"total_count": count, "items": user_schema.dump(users)}

    @classmethod
    def get_all_external_json(cls, user, search):
        users, count = cls.get(search, user.organizations[0])
        user_schema = UserPresentationSchema(many=True)
        return {"total_count": count, "items": user_schema.dump(users)}

    @classmethod
    def add_new(cls, data):
        new_user_schema = NewUserSchema()
        user = new_user_schema.load(data)
        db.session.add(user)
        db.session.commit()

    @classmethod
    def add_new_external(cls, user, permissions, data):
        new_user_schema = NewUserSchema()
        new_user = new_user_schema.load(data)
        new_user.roles = []
        new_user.organizations = user.organizations

        for permission in new_user.permissions[:]:
            if permission.id not in permissions:
                new_user.permissions.remove(permission)

        db.session.add(new_user)
        db.session.commit()

    @classmethod
    def update(cls, user_id, data):
        schema = NewUserSchema()
        updated_user = schema.load(data)
        user = cls.query.get(user_id)
        user.username = updated_user.username
        user.name = updated_user.name
        user.organizations = updated_user.organizations
        user.roles = updated_user.roles
        user.permissions = updated_user.permissions
        db.session.commit()

    @classmethod
    def update_external(cls, user, permissions, user_id, data):
        schema = NewUserSchema()
        updated_user = schema.load(data)
        existing_user = cls.query.get(user_id)

        if any(org in user.organizations for org in existing_user.organizations):
            existing_user.username = updated_user.username
            existing_user.name = updated_user.name

            for permission in updated_user.permissions[:]:
                if permission.id not in permissions:
                    updated_user.permissions.remove(permission)

            existing_user.permissions = updated_user.permissions

            db.session.commit()

    @classmethod
    def delete(cls, id):
        user = cls.query.get(id)
        db.session.delete(user)
        db.session.commit()

    @classmethod
    def delete_external(cls, user, id):
        existing_user = cls.query.get(id)
        if any(org in user.organizations for org in existing_user.organizations):
            db.session.delete(existing_user)
            db.session.commit()

    def get_permissions(self):
        all_permissions = {permission.id for permission in self.permissions}

        for role in self.roles:
            all_permissions.update(role.get_permissions())
        return list(all_permissions)

    def get_current_organization_name(self):
        return self.organizations[0].name if len(self.organizations) > 0 else ""

    @classmethod
    def get_profile_json(cls, user):
        profile_schema = UserProfileSchema()
        return profile_schema.dump(user.profile)

    @classmethod
    def update_profile(cls, user, data):
        new_profile_schema = NewUserProfileSchema()
        updated_profile = new_profile_schema.load(data)

        user.profile.spellcheck = updated_profile.spellcheck
        user.profile.dark_theme = updated_profile.dark_theme

        user.profile.hotkeys = updated_profile.hotkeys

        db.session.commit()

        return cls.get_profile_json(user)


class UserOrganization(db.Model):
    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), primary_key=True)
    organization_id = db.Column(db.Integer, db.ForeignKey("organization.id"), primary_key=True)


class UserRole(db.Model):
    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), primary_key=True)
    role_id = db.Column(db.Integer, db.ForeignKey("role.id"), primary_key=True)


class UserPermission(db.Model):
    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), primary_key=True)
    permission_id = db.Column(db.String, db.ForeignKey("permission.id"), primary_key=True)


class NewHotkeySchema(HotkeySchema):
    @post_load
    def make(self, data, **kwargs):
        return Hotkey(**data)


class NewUserProfileSchema(UserProfileSchema):
    hotkeys = fields.List(fields.Nested(NewHotkeySchema))

    @post_load
    def make(self, data, **kwargs):
        return UserProfile(**data)


class UserProfile(db.Model):
    id = db.Column(db.Integer, primary_key=True)

    spellcheck = db.Column(db.Boolean, default=True)
    dark_theme = db.Column(db.Boolean, default=False)

    hotkeys = db.relationship("Hotkey", cascade="all, delete-orphan")

    def __init__(self, spellcheck, dark_theme, hotkeys):
        self.id = None
        self.spellcheck = spellcheck
        self.dark_theme = dark_theme
        self.hotkeys = hotkeys


class UserProfileWordList(db.Model):
    user_profile_id = db.Column(db.Integer, db.ForeignKey("user_profile.id"), primary_key=True)


class Hotkey(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    key_code = db.Column(db.Integer)
    key = db.Column(db.String)
    alias = db.Column(db.String)

    user_profile_id = db.Column(db.Integer, db.ForeignKey("user_profile.id"))

    def __init__(self, key_code, key, alias):
        self.id = None
        self.key_code = key_code
        self.key = key
        self.alias = alias



from marshmallow import fields, post_load
import uuid

from core.managers.db_manager import db
from core.model.parameter import NewParameterSchema
from shared.schema.bot import BotSchema


class NewBotSchema(BotSchema):
    parameters = fields.List(fields.Nested(NewParameterSchema))

    @post_load
    def make(self, data, **kwargs):
        return Bot(**data)


class Bot(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    type = db.Column(db.String(64), nullable=False)

    parameters = db.relationship("Parameter", secondary="bot_parameter")

    node_id = db.Column(db.String, db.ForeignKey("bots_node.id"))
    node = db.relationship("BotsNode", back_populates="bots")

    presets = db.relationship("BotPreset", back_populates="bot")

    def __init__(self, name, description, type, parameters):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.type = type
        self.parameters = parameters

    @classmethod
    def create_all(cls, bots_data):
        new_bot_schema = NewBotSchema(many=True)
        return new_bot_schema.load(bots_data)


class BotParameter(db.Model):
    bot_id = db.Column(db.String, db.ForeignKey("bot.id"), primary_key=True)
    parameter_id = db.Column(db.Integer, db.ForeignKey("parameter.id"), primary_key=True)



import threading

from core.managers.db_manager import db
from core.managers import publishers_manager
from core.model.asset import Asset
from core.model.publisher_preset import PublisherPreset


def remove_vulnerability(report_item_id):
    Asset.remove_vulnerability(report_item_id)
    db.session.commit()


def report_item_changed(report_item):
    if report_item.completed:
        cpes = []
        for cpe in report_item.report_item_cpes:
            cpes.append(cpe.value)

        assets = Asset.get_by_cpe(cpes)

        notification_groups = set()

        for asset in assets:
            asset.add_vulnerability(report_item)
            notification_groups.add(asset.asset_group)

        db.session.commit()

        publisher_preset = PublisherPreset.find_for_notifications()
        if publisher_preset is not None:

            class NotificationThread(threading.Thread):
                @classmethod
                def run(cls):
                    for notification_group in notification_groups:
                        for template in notification_group.templates:
                            recipients = [recipient.email for recipient in template.recipients]
                            publishers_manager.publish(
                                publisher_preset,
                                None,
                                template.message_title,
                                template.message_body,
                                recipients,
                            )

            notification_thread = NotificationThread()
            notification_thread.start()



from os import environ
from flask_jwt_extended import get_jwt
from flask_restful import request
from requests import post
from requests.auth import HTTPBasicAuth

from core.managers.log_manager import logger
from core.auth.base_authenticator import BaseAuthenticator


class KeycloakAuthenticator(BaseAuthenticator):
    def authenticate(self, credentials):

        # check if code and session_state are present in keycloak callback
        if "code" not in request.args or "session_state" not in request.args:
            return {"error": "Missing code or session_state parameters"}, 400

        # verify code and get JWT token from keycloak
        response = post(
            url=environ.get("TARANIS_NG_KEYCLOAK_INTERNAL_URL") + "auth/realms/taranis_ng/protocol/openid-connect/token",
            data={
                "grant_type": "authorization_code",
                "code": request.args["code"],  # code from url
                "redirect_uri": "/".join(request.headers.get("Referer").split("/")[0:3]) + "/login"
                # original redirect_uri (host needs to match)
            },
            auth=HTTPBasicAuth(
                environ.get("TARANIS_NG_KEYCLOAK_CLIENT_ID"),
                environ.get("TARANIS_NG_KEYCLOAK_CLIENT_SECRET"),
            ),
            # do not forget credentials
            proxies={"http": None, "https": None},
            allow_redirects=False,
        )

        data = None

        try:
            # get json data from response
            data = response.json()
        except Exception:
            logger.store_auth_error_activity("Keycloak returned an unexpected response.")
            return {"error": "Internal server error"}, 500

        try:
            # decode token to get user data
            data = get_jwt()["access_token"]
            user = auth_manager.get_user_from_jwt()
        except Exception:
            logger.store_auth_error_activity("Keycloak returned invalid access_token.")
            return {"error": "Internal server error"}, 500

        # generate custom token
        return BaseAuthenticator.generate_jwt(data["preferred_username"])



import os
from datetime import datetime, timedelta
from enum import Enum, auto
from functools import wraps
from flask import request
from flask_jwt_extended import (
    JWTManager,
    get_jwt,
    get_jwt_identity,
    verify_jwt_in_request,
)
from flask_jwt_extended.exceptions import JWTExtendedException

from core.managers import time_manager
from core.managers.log_manager import logger
from core.auth.keycloak_authenticator import KeycloakAuthenticator
from core.auth.openid_authenticator import OpenIDAuthenticator
from core.auth.test_authenticator import TestAuthenticator
from core.auth.database_authenticator import DatabaseAuthenticator
from core.model.collectors_node import CollectorsNode
from core.model.news_item import NewsItem
from core.model.osint_source import OSINTSourceGroup
from core.model.permission import Permission
from core.model.product_type import ProductType
from core.model.remote import RemoteAccess
from core.model.report_item import ReportItem
from core.model.token_blacklist import TokenBlacklist
from core.model.user import User
from core.config import Config

current_authenticator = None

api_key = Config.API_KEY


def cleanup_token_blacklist(app):
    with app.app_context():
        TokenBlacklist.delete_older(datetime.now() - timedelta(days=1))


def initialize(app):
    global current_authenticator

    JWTManager(app)

    which = os.getenv("TARANIS_NG_AUTHENTICATOR")
    if which == "openid":
        current_authenticator = OpenIDAuthenticator()
    elif which == "keycloak":
        current_authenticator = KeycloakAuthenticator()
    elif which == "database":
        current_authenticator = DatabaseAuthenticator()
    elif which == "test":
        current_authenticator = TestAuthenticator()
    else:
        current_authenticator = TestAuthenticator()

    with app.app_context():
        current_authenticator.initialize(app)

    time_manager.schedule_job_every_day("00:00", cleanup_token_blacklist, app)


def get_required_credentials():
    return current_authenticator.get_required_credentials()


def authenticate(credentials):
    logger.log_debug(f"credentials: {credentials}")
    return current_authenticator.authenticate(credentials)


def refresh(user):
    return current_authenticator.refresh(user)


def logout(token):
    return current_authenticator.logout(token)


class ACLCheck(Enum):
    OSINT_SOURCE_GROUP_ACCESS = auto()
    NEWS_ITEM_ACCESS = auto()
    NEWS_ITEM_MODIFY = auto()
    REPORT_ITEM_ACCESS = auto()
    REPORT_ITEM_MODIFY = auto()
    PRODUCT_TYPE_ACCESS = auto()
    PRODUCT_TYPE_MODIFY = auto()


def check_acl(item_id, acl_check, user):
    check_see = "SEE" in str(acl_check)
    check_access = "ACCESS" in str(acl_check)
    check_modify = "MODIFY" in str(acl_check)
    allowed = False
    item_type = "UNKNOWN"

    if acl_check == ACLCheck.OSINT_SOURCE_GROUP_ACCESS:
        item_type = "OSINT Source Group"
        allowed = OSINTSourceGroup.allowed_with_acl(item_id, user, check_see, check_access, check_modify)

    if acl_check in [ACLCheck.NEWS_ITEM_ACCESS, ACLCheck.NEWS_ITEM_MODIFY]:
        item_type = "News Item"
        allowed = NewsItem.allowed_with_acl(item_id, user, check_see, check_access, check_modify)

    if acl_check in [ACLCheck.REPORT_ITEM_ACCESS, ACLCheck.REPORT_ITEM_MODIFY]:
        item_type = "Report Item"
        allowed = ReportItem.allowed_with_acl(item_id, user, check_see, check_access, check_modify)

    if acl_check in [ACLCheck.PRODUCT_TYPE_ACCESS, ACLCheck.PRODUCT_TYPE_MODIFY]:
        item_type = "Product Type"
        allowed = ProductType.allowed_with_acl(item_id, user, check_see, check_access, check_modify)

    if not allowed:
        logger.log_debug(f"Unauthorized '{check_access or check_modify}' attempt to {item_type}: {item_id}")

    return allowed


def no_auth(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        logger.store_activity("API_ACCESS", None)
        return fn(*args, **kwargs)

    return wrapper


def get_id_name_by_acl(acl):
    if "NEWS_ITEM" in acl.name:
        return "item_id"
    elif "REPORT_ITEM" in acl.name:
        return "report_item_id"
    elif "OSINT_SOURCE_GROUP" in acl.name:
        return "group_id"
    elif "PRODUCT" in acl.name:
        return "product_id"


def auth_required(permissions, acl=None):
    def auth_required_wrap(fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            error = ({"error": "not authorized"}, 401)
            permissions_set = set(permissions) if isinstance(permissions, list) else {permissions}

            # do we have a JWT token?
            try:
                verify_jwt_in_request()
            except JWTExtendedException:
                logger.store_auth_error_activity("Missing JWT")
                return error

            # does it encode an identity?
            identity = get_jwt_identity()
            if not identity:
                logger.store_auth_error_activity(f"Missing identity in JWT: {get_jwt()}")
                return error

            user = User.find(identity)

            # does it include permissions?
            claims = get_jwt()
            user_claims = claims.get("user_claims")
            if not user_claims:
                logger.store_user_auth_error_activity(user, "Missing permissions in JWT for identity: {}".format(identity))
                return error

            permission_claims = set(user_claims.get("permissions"))

            # is there at least one match with the permissions required by the call?
            if not permissions_set.intersection(permission_claims):
                logger.store_user_auth_error_activity(
                    user,
                    "Insufficient permissions in JWT for identity: {}".format(identity),
                )
                return error

            # if the object does have an ACL, do we match it?
            if acl and not check_acl(kwargs[get_id_name_by_acl(acl)], acl, user):
                logger.store_user_auth_error_activity(
                    user,
                    "Access denied by ACL in JWT for identity: {}".format(identity),
                )
                return error

            return fn(*args, **kwargs)

        return wrapper

    return auth_required_wrap


def api_key_required(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        error = ({"error": "not authorized"}, 401)

        # do we have the authorization header?
        auth_header = request.headers.get("Authorization", None)
        if not auth_header:
            logger.store_auth_error_activity("Missing Authorization header for external access")
            return error

        if not auth_header.startswith("Bearer"):
            logger.store_auth_error_activity("Missing Authorization Bearer for external access")
            return error

        api_key = auth_header.replace("Bearer ", "")

        # does it match some of our collector's keys?
        if not CollectorsNode.exists_by_api_key(api_key) and Config.API_KEY != api_key:
            logger.store_auth_error_activity(f"Incorrect api key: {api_key} for external access")
            return error

        # allow
        return fn(*args, **kwargs)

    return wrapper


def access_key_required(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        error = ({"error": "not authorized"}, 401)

        auth_header = request.headers.get("Authorization", None)
        if not auth_header:
            logger.store_auth_error_activity("Missing Authorization header for remote access")
            return error

        if not auth_header.startswith("Bearer"):
            logger.store_auth_error_activity("Missing Authorization Bearer for remote access")
            return error

        # does it match some of our remote peer's access keys?
        if not RemoteAccess.exists_by_access_key(auth_header.replace("Bearer ", "")):
            logger.store_auth_error_activity("Incorrect access key: " + auth_header.replace("Bearer ", "") + " for remote access")
            return error

        # allow
        return fn(*args, **kwargs)

    return wrapper


def get_access_key():
    return request.headers["Authorization"].replace("Bearer ", "")


def get_user_from_jwt():
    try:
        verify_jwt_in_request()
    except JWTExtendedException:
        return None
    identity = get_jwt_identity()

    return User.find(identity) if identity else None


def get_external_permissions_ids():
    return ["MY_ASSETS_ACCESS", "MY_ASSETS_CREATE", "MY_ASSETS_CONFIG"]


def get_external_permissions():
    permissions = []
    for permission_id in get_external_permissions_ids():
        permissions.append(Permission.find(permission_id))

    return permissions



import datetime
from marshmallow import post_load, fields
from sqlalchemy import func, or_, orm, and_
import sqlalchemy
from sqlalchemy.sql.expression import cast

from core.managers.db_manager import db
from core.model.product import Product
from core.model.parameter_value import NewParameterValueSchema
from core.model.acl_entry import ACLEntry
from shared.schema.acl_entry import ItemType
from shared.schema.product_type import ProductTypePresentationSchema, ProductTypeSchema


class NewProductTypeSchema(ProductTypeSchema):
    parameter_values = fields.List(fields.Nested(NewParameterValueSchema))

    @post_load
    def make(self, data, **kwargs):
        return ProductType(**data)


class ProductType(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(64), unique=True, nullable=False)
    description = db.Column(db.String(), nullable=False)

    created = db.Column(db.DateTime, default=datetime.datetime.utcnow)

    presenter_id = db.Column(db.String, db.ForeignKey("presenter.id"))
    presenter = db.relationship("Presenter")

    parameter_values = db.relationship("ParameterValue", secondary="product_type_parameter_value", cascade="all")

    def __init__(self, id, title, description, presenter_id, parameter_values):
        self.id = None
        self.title = title
        self.description = description
        self.presenter_id = presenter_id
        self.parameter_values = parameter_values
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.subtitle = self.description
        self.tag = "mdi-file-document-outline"

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(ProductType.title)).all()

    @classmethod
    def allowed_with_acl(cls, product_id, user, see, access, modify):
        product = db.session.query(Product).filter_by(id=product_id).first()
        if not product:
            return False

        query = db.session.query(ProductType.id).distinct().group_by(ProductType.id).filter(ProductType.id == product.product_type_id)

        query = query.outerjoin(
            ACLEntry,
            and_(
                cast(ProductType.id, sqlalchemy.String) == ACLEntry.item_id,
                ACLEntry.item_type == ItemType.PRODUCT_TYPE,
            ),
        )

        query = ACLEntry.apply_query(query, user, see, access, modify)

        return query.scalar() is not None

    @classmethod
    def get(cls, search, user, acl_check):
        query = cls.query.distinct().group_by(ProductType.id)

        if acl_check:
            query = query.outerjoin(
                ACLEntry,
                and_(
                    cast(ProductType.id, sqlalchemy.String) == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.PRODUCT_TYPE,
                ),
            )
            query = ACLEntry.apply_query(query, user, True, False, False)

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(ProductType.title).like(search_string),
                    func.lower(ProductType.description).like(search_string),
                )
            )

        return query.order_by(db.asc(ProductType.title)).all(), query.count()

    @classmethod
    def get_all_json(cls, search, user, acl_check):
        product_types, count = cls.get(search, user, acl_check)
        product_type_schema = ProductTypePresentationSchema(many=True)
        return {"total_count": count, "items": product_type_schema.dump(product_types)}

    @classmethod
    def add_new(cls, data):
        new_product_type_schema = NewProductTypeSchema()
        product_type = new_product_type_schema.load(data)
        db.session.add(product_type)
        db.session.commit()

    @classmethod
    def delete(cls, id):
        product_type = cls.query.get(id)
        db.session.delete(product_type)
        db.session.commit()

    @classmethod
    def update(cls, preset_id, data):
        new_product_type_schema = NewProductTypeSchema()
        updated_product_type = new_product_type_schema.load(data)
        product_type = cls.query.get(preset_id)
        product_type.title = updated_product_type.title
        product_type.description = updated_product_type.description

        for value in product_type.parameter_values:
            for updated_value in updated_product_type.parameter_values:
                if value.parameter_id == updated_value.parameter_id:
                    value.value = updated_value.value

        db.session.commit()


class ProductTypeParameterValue(db.Model):
    product_type_id = db.Column(db.Integer, db.ForeignKey("product_type.id"), primary_key=True)
    parameter_value_id = db.Column(db.Integer, db.ForeignKey("parameter_value.id"), primary_key=True)



from flask import request

from core.model.audit_record import AuditRecord


def resolve_ip_address():
    headers_list = request.headers.getlist("X-Forwarded-For")
    ip_address = headers_list[0] if headers_list else request.remote_addr
    return ip_address


def resolve_resource():
    fp_len = len(request.full_path)
    if request.full_path[fp_len - 1] == "?":
        return request.full_path[: fp_len - 1]
    else:
        return request.full_path.split("?")[0]


def store_activity(activity_type, activity_detail):
    AuditRecord.store(
        resolve_ip_address(),
        None,
        None,
        None,
        None,
        activity_type,
        resolve_resource(),
        activity_detail,
    )


def store_user_activity(user, activity_type, activity_detail):
    AuditRecord.store(
        resolve_ip_address(),
        user.id,
        user.name,
        None,
        None,
        activity_type,
        resolve_resource(),
        activity_detail,
    )


def store_auth_error_activity(activity_detail):
    AuditRecord.store(
        resolve_ip_address(),
        None,
        None,
        None,
        None,
        "AUTH_ERROR",
        resolve_resource(),
        activity_detail,
    )


def store_user_auth_error_activity(user, activity_detail):
    AuditRecord.store(
        resolve_ip_address(),
        user.id,
        user.name,
        None,
        None,
        "AUTH_ERROR",
        resolve_resource(),
        activity_detail,
    )


def store_system_activity(system_id, system_name, activity_type, activity_detail):
    AuditRecord.store(
        resolve_ip_address(),
        None,
        None,
        system_id,
        system_name,
        activity_type,
        resolve_resource(),
        activity_detail,
    )



import datetime
import re
from marshmallow import post_load
from sqlalchemy import func
from sqlalchemy.sql import label

from core.managers.db_manager import db
from shared.schema.tag_cloud import TagCloudSchema, GroupedWordsSchema


class NewTagCloudSchema(TagCloudSchema):
    @post_load
    def make_tag_cloud(self, data, **kwargs):
        return TagCloud(**data)


class TagCloud(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    word = db.Column(db.String())
    word_quantity = db.Column(db.BigInteger)
    collected = db.Column(db.Date)

    def __init__(self, word, word_quantity, collected):
        self.id = None
        self.word = word
        self.word_quantity = word_quantity
        self.collected = collected

    @classmethod
    def add_tag_clouds(cls, tag_clouds):
        for tag_cloud in tag_clouds:
            word = TagCloud.query.filter_by(word=tag_cloud.word, collected=tag_cloud.collected).first()
            if word is not None:
                word.word_quantity += 1
            else:
                db.session.add(tag_cloud)
        db.session.commit()

    @classmethod
    def get_grouped_words(cls, tag_cloud_day):
        day_filter = (datetime.datetime.now() - datetime.timedelta(days=tag_cloud_day)).date()
        grouped_words = (
            db.session.query(TagCloud.word, label("word_quantity", func.sum(TagCloud.word_quantity)))
            .filter(TagCloud.collected == day_filter)
            .group_by(TagCloud.word)
            .order_by(db.desc("word_quantity"))
            .limit(100)
            .all()
        )
        grouped_words_schema = GroupedWordsSchema(many=True)
        return grouped_words_schema.dump(grouped_words)

    @classmethod
    def delete_words(cls):
        limit_days = 7
        limit = (datetime.datetime.now() - datetime.timedelta(days=limit_days)).date()
        cls.query.filter(cls.collected < limit).delete()
        db.session.commit()

    @staticmethod
    def unwanted_chars(news_item_data):
        title = news_item_data.title.lower()
        title = re.sub(r"[^a-zA-Z0-9 ]", r"", title)
        review = news_item_data.review.lower()
        review = re.sub(r"[^a-zA-Z0-9 ]", r"", review)
        content = news_item_data.content.lower()
        content = re.sub(r"[^a-zA-Z0-9 ]", r"", content)
        return title, review, content

    @staticmethod
    def create_tag_cloud(word):
        collected = datetime.datetime.now().date()
        tag_cloud_word = TagCloud(word, 1, collected)
        return tag_cloud_word

    @staticmethod
    def news_item_words(title, review, content):
        news_item_title_words = title.split()
        news_item_review_words = review.split()
        news_item_content_words = content.split()
        return news_item_title_words, news_item_review_words, news_item_content_words

    @staticmethod
    def news_items_words(
        title,
        review,
        content,
        news_items_title_words,
        news_items_review_words,
        news_items_content_words,
    ):
        (
            news_item_title_words,
            news_item_review_words,
            news_item_content_words,
        ) = TagCloud.news_item_words(title, review, content)
        news_items_title_words.extend(news_item_title_words)
        news_items_review_words.extend(news_item_review_words)
        news_items_content_words.extend(news_item_content_words)
        return news_items_title_words, news_items_review_words, news_items_content_words

    @classmethod
    def generate_tag_cloud_words(cls, news_item_data):

        news_items_title_words = []
        news_items_review_words = []
        news_items_content_words = []
        tag_cloud_words = []

        title, review, content = TagCloud.unwanted_chars(news_item_data)

        (news_items_title_words, news_items_review_words, news_items_content_words,) = TagCloud.news_items_words(
            title,
            review,
            content,
            news_items_title_words,
            news_items_review_words,
            news_items_content_words,
        )
        news_items_words = news_items_title_words + news_items_review_words + news_items_content_words

        for word_item in set(news_items_words):
            tag_cloud_words.append(TagCloud.create_tag_cloud(word_item))

        cls.add_tag_clouds(tag_cloud_words)



from marshmallow import fields, post_load
import uuid

from core.managers.db_manager import db
from core.model.parameter import NewParameterSchema
from shared.schema.collector import CollectorSchema


class NewCollectorSchema(CollectorSchema):
    parameters = fields.List(fields.Nested(NewParameterSchema))

    @post_load
    def make_collector(self, data, **kwargs):
        return Collector(**data)


class Collector(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    type = db.Column(db.String(), nullable=False)

    parameters = db.relationship("Parameter", secondary="collector_parameter", cascade="all")

    node_id = db.Column(db.String, db.ForeignKey("collectors_node.id", ondelete="CASCADE"))
    node = db.relationship("CollectorsNode", back_populates="collectors")

    sources = db.relationship("OSINTSource", back_populates="collector")

    def __init__(self, name, description, type, parameters):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.type = type
        self.parameters = parameters

    @classmethod
    def create_all(cls, collectors_data):
        new_collector_schema = NewCollectorSchema(many=True)
        return new_collector_schema.load(collectors_data)


class CollectorParameter(db.Model):
    collector_id = db.Column(db.String, db.ForeignKey("collector.id", ondelete="CASCADE"), primary_key=True)
    parameter_id = db.Column(db.Integer, db.ForeignKey("parameter.id", ondelete="CASCADE"), primary_key=True)



import json
from requests.exceptions import ConnectionError

from core.model.collector import Collector
from core.model.collectors_node import CollectorsNode
from core.model.osint_source import OSINTSource
from core.remote.collectors_api import CollectorsApi
from core.managers.log_manager import logger
from shared.schema.collectors_node import CollectorsNode as CollectorNodeSchema
from shared.schema.osint_source import OSINTSourceExportRootSchema, OSINTSourceExportRoot


def get_collectors_info(node: CollectorNodeSchema):
    try:
        collectors_info, status_code = CollectorsApi(node.api_url, node.api_key).get_collectors_info()
    except ConnectionError:
        return f"Connection error: Could not reach {node.api_url}", 500
    except Exception:
        logger.log_debug_trace(f"Couldn't add Collector Node: {node.name}")
        return f"Couldn't add Collector node: {node.name}", 500

    if status_code != 200:
        return None, status_code

    return Collector.create_all(collectors_info), status_code


def add_collectors_node(data):
    try:
        collectors_info = data.pop("collectors_info")
        logger.log_debug(f"ADD COLLECTOR: {data}")
        node = CollectorNodeSchema.create(data)

    except Exception as e:
        logger.log_debug_trace()
        return str(e), 500

    try:
        collectors = Collector.create_all(collectors_info)
        CollectorsNode.add_new(data, collectors)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Collector Node: {node.name}")
        return f"Couldn't add Collector node: {node.name}", 500

    return node.id, 200


def update_collectors_node(node_id, data):
    node = CollectorNodeSchema.create(data)
    collectors, status_code = get_collectors_info(node)
    if status_code != 200:
        return collectors, status_code

    try:
        CollectorsNode.update(node_id, data, collectors)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Collector Node: {node.name}")
        return f"Couldn't add Collector node: {node.name}", 500

    return node.id, status_code


def add_osint_source(data):
    osint_source = OSINTSource.add_new(data)
    refresh_collector(osint_source.collector)


def update_osint_source(osint_source_id, data):
    osint_source, default_group = OSINTSource.update(osint_source_id, data)
    refresh_collector(osint_source.collector)
    return osint_source, default_group


def delete_osint_source(osint_source_id):
    osint_source = OSINTSource.find(osint_source_id)
    collector = osint_source.collector
    OSINTSource.delete(osint_source_id)
    refresh_collector(collector)


def refresh_collector(collector):
    return CollectorsApi(collector.node.api_url, collector.node.api_key).refresh_collector(collector.type)


def export_osint_sources(input_data):
    osint_sources = OSINTSource.get_all()
    if input_data is not None and "selection" in input_data:
        data = []
        for osint_source in osint_sources[:]:
            if osint_source.id in input_data["selection"]:
                data.append(osint_source)
    else:
        data = osint_sources

    schema = OSINTSourceExportRootSchema()
    export_data = schema.dump(OSINTSourceExportRoot(1, data))

    for osint_source in export_data["data"]:
        for parameter_value in osint_source["parameter_values"]:
            if parameter_value["parameter"]["key"] == "PROXY_SERVER":
                parameter_value["value"] = ""

    return json.dumps(export_data).encode("utf-8")


def import_osint_sources(collectors_node_id, file):
    collectors_node = CollectorsNode.get_by_id(collectors_node_id)

    file_data = file.read()
    json_data = json.loads(file_data.decode("utf8"))
    schema = OSINTSourceExportRootSchema()
    import_data = schema.load(json_data)

    collectors = set()
    for osint_source in import_data.data:
        collector = collectors_node.find_collector_by_type(osint_source.collector.type)
        if collector is not None:
            collectors.add(collector)
            OSINTSource.import_new(osint_source, collector)

    for collector in collectors:
        refresh_collector(collector)



from core.model.presenters_node import PresentersNode
from core.model.presenter import Presenter
from core.model.product import Product
from core.remote.presenters_api import PresentersApi
from core.managers.log_manager import logger
from shared.schema.presenters_node import PresentersNode as PresentersNodeSchema
from shared.schema.presenter import PresenterInput, PresenterInputSchema


def get_presenters_info(node: PresentersNodeSchema):
    try:
        presenters_info, status_code = PresentersApi(node.api_url, node.api_key).get_presenters_info()
    except ConnectionError:
        return f"Connection error: Could not reach {node.api_url}", 500
    except Exception:
        logger.log_debug_trace(f"Couldn't add Presenter Node: {node.name}")
        return f"Couldn't add Presenter node: {node.name}", 500

    if status_code != 200:
        return None, status_code

    return Presenter.create_all(presenters_info), status_code


def add_presenters_node(data):
    try:
        logger.log_info(data)
        presenters_info = data.pop("presenters_info")
        node = PresentersNodeSchema.create(data)
    except Exception as e:
        logger.log_debug_trace()
        return str(e), 500

    try:
        presenters = Presenter.create_all(presenters_info)
        PresentersNode.add_new(data, presenters)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Presenter Node: {node.name}")
        return f"Couldn't add Presenter Node: {node.name}", 500

    return node.id, 200


def update_presenters_node(node_id, data):
    node = PresentersNodeSchema.create(data)
    presenters, status_code = get_presenters_info(node)

    if status_code != 200:
        return presenters, status_code

    try:
        PresentersNode.update(node_id, data, presenters)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Presenter Node: {node.name}")
        return f"Couldn't add Presenter node: {node.name}", 500

    return node.id, status_code


def generate_product(product_id):
    product = Product.find(product_id)
    presenter = product.product_type.presenter
    node = presenter.node

    input_data = PresenterInput(
        presenter.type,
        product.product_type.parameter_values,
        product.report_items,
        product.report_items[0].report_item_type,
    )
    input_schema = PresenterInputSchema()

    return PresentersApi(node.api_url, node.api_key).generate(input_schema.dump(input_data))



from flask import Flask
from flask_cors import CORS
from core.managers import (
    db_manager,
    auth_manager,
    api_manager,
    sse_manager,
    remote_manager,
    tagcloud_manager,
    log_manager,
)


def create_app():
    app = Flask(__name__)
    app.config.from_object("core.config.Config")

    with app.app_context():
        CORS(app)

        log_manager.logger.log_info(f"Connecting Database: {app.config.get('SQLALCHEMY_DATABASE_URI')}")
        db_manager.initialize(app)
        log_manager.logger.log_info("DB Done")

        auth_manager.initialize(app)
        api_manager.initialize(app)

        sse_manager.initialize(app)
        remote_manager.initialize(app)
        tagcloud_manager.initialize(app)
        log_manager.logger.log_info("All Done")
        # import test

    return app



__all__ = [
    "analyze",
    "assess",
    "assets",
    "auth",
    "bots",
    "collectors",
    "config",
    "dashboard",
    "isalive",
    "openapi",
    "presenters",
    "publish",
    "publishers",
    "remote",
    "user",
]

import core.api.analyze
import core.api.assess
import core.api.assets
import core.api.auth
import core.api.bots
import core.api.collectors
import core.api.config
import core.api.dashboard
import core.api.isalive
import core.api.openapi
import core.api.presenters
import core.api.publish
import core.api.publishers
import core.api.remote
import core.api.user  # noqa



from flask import request, abort
from flask_sse import sse
from datetime import datetime
import os

from core.managers import auth_manager, bots_manager, time_manager, remote_manager
from core.model.remote import RemoteAccess


def use_sse():
    return os.getenv("REDIS_URL") is not None


def news_items_updated():
    if use_sse():
        sse.publish({}, type="news-items-updated")


def report_items_updated():
    if use_sse():
        sse.publish({}, type="report-items-updated")


def report_item_updated(data):
    if use_sse():
        sse.publish(data, type="report-item-updated")


def remote_access_disconnect(data):
    if use_sse():
        sse.publish(data, type="remote_access_disconnect", channel="remote")


def remote_access_news_items_updated(osint_source_ids):
    if use_sse():
        remote_access_event_ids = RemoteAccess.get_relevant_for_news_items(osint_source_ids)
        sse.publish(
            remote_access_event_ids,
            type="remote_access_news_items_updated",
            channel="remote",
        )


def remote_access_report_items_updated(report_item_type_id):
    if use_sse():
        remote_access_event_ids = RemoteAccess.get_relevant_for_report_item(report_item_type_id)
        sse.publish(
            remote_access_event_ids,
            type="remote_access_report_items_updated",
            channel="remote",
        )


report_item_locks = {}
report_item_locks_last_check_time = datetime.now()


@sse.before_request
def connect():
    if request.args.get("jwt") is not None:
        if auth_manager.decode_user_from_jwt(request.args.get("jwt")) is None:
            abort(403)
    elif request.args.get("api_key") is not None:
        if bots_manager.verify_api_key(request.args.get("api_key")) is False:
            abort(403)
    elif request.args.get("access_key") is not None and request.args.get("channel") is not None:
        if request.args.get("channel") == "remote":
            if remote_manager.verify_access_key(request.args.get("access_key")) is False:
                abort(403)
    else:
        abort(403)


def report_item_lock(report_item_id, field_id, user_id):
    if report_item_id in report_item_locks:
        report_item = report_item_locks[report_item_id]
    else:
        report_item = {}
        report_item_locks[report_item_id] = report_item

    if field_id not in report_item or report_item[field_id] is None:
        report_item[field_id] = {"user_id": user_id, "lock_time": datetime.now()}
        sse.publish(
            {
                "report_item_id": int(report_item_id),
                "field_id": field_id,
                "user_id": user_id,
            },
            type="report-item-locked",
        )


def report_item_unlock(report_item_id, field_id, user_id):
    if report_item_id in report_item_locks:
        report_item = report_item_locks[report_item_id]

        if field_id in report_item:
            report_item[field_id] = None

    sse.publish(
        {
            "report_item_id": int(report_item_id),
            "field_id": field_id,
            "user_id": user_id,
        },
        type="report-item-unlocked",
    )


def report_item_hold_lock(report_item_id, field_id, user_id):
    if report_item_id in report_item_locks:
        report_item = report_item_locks[report_item_id]
        if field_id in report_item and report_item[field_id] is not None:
            if report_item[field_id]["user_id"] == user_id:
                report_item[field_id]["lock_time"] = datetime.now()


def check_report_item_locks(app):
    global report_item_locks_last_check_time
    for key in report_item_locks:
        for field_key in report_item_locks[key]:
            if report_item_locks[key][field_key] is not None:
                if report_item_locks[key][field_key]["lock_time"] < report_item_locks_last_check_time:
                    report_item_locks[key][field_key] = None
                    with app.app_context():
                        sse.publish(
                            {
                                "report_item_id": int(key),
                                "field_id": field_key,
                                "user_id": -1,
                            },
                            type="report-item-unlocked",
                        )

    report_item_locks_last_check_time = datetime.now()


def initialize(app):
    time_manager.schedule_job_minutes(1, check_report_item_locks, app)
